/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Benjamin Grauer
   co-programmer: ...
*/

#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_LOAD

#include "resource.h"
#include "filesys/file.h"
#include "debug.h"

ObjectListDefinition(NewResource);
std::vector<NewResource::Type*>    NewResource::_resourceTypes;

//! GLOBALS
Directory NewResource::_mainGlobalPath;
std::vector<Directory>        NewResource::_globalPaths;


/**
 * standard constructor
*/
NewResource::NewResource (NewResource::Type* type)
    : _pointer(NULL), _type(type)
{
  this->registerObject(this, NewResource::_objectList);
}

/**
 * standard deconstructor
 */
NewResource::~NewResource ()
{
  // delete what has to be deleted here
}


std::string NewResource::locateFile(const std::string& fileName) const
{
  if (File(fileName).exists())
    return fileName;
  else if ((NewResource::_mainGlobalPath + File(fileName)).exists() )
    return (NewResource::_mainGlobalPath + File(fileName)).name();

  printf("LOCATED %s\n", locateFileInSubDir(Directory("/home/bensch/svn/orxonox/data/"), fileName).c_str());

  return std::string("/home/bensch/svn/orxonox/data/") + fileName;
}

/**
 * @brief tests in all the SubDirectories defined in Resource under Directory if the fileName exists.
 * @param directory the directory to in what to search for all subdirectories for.
 * @param fileName the Name of the File to query for
 * @return true on success.
 */
std::string NewResource::locateFileInSubDir(const Directory& directory, const std::string& fileName) const
{
  std::vector<Directory>::const_iterator it;
  for (it = this->_type->resourceSubPaths().begin(); it != this->_type->resourceSubPaths().end(); ++it)
  {
    Directory dir = directory + (*it);
    File file = dir + File(fileName);
    printf("Testing %s (from %s in directory %s)\n", file.name().c_str(), fileName.c_str(), dir.name().c_str());
    if ((dir+ File(fileName)).exists())
      return (dir+File(fileName)).name();
  }
  return "";
}



NewResource::StorePointer* NewResource::acquireResource(const std::string& loadString)
{
  //const NewResource::Type* const type = NewResource::_resourceTypes[this->_type->id()];

  for (unsigned int i = 0; i < _type->storedResources().size(); ++i)
  {
    if (_type->storedResources()[i]->loadString() == loadString)
      return _type->storedResources()[i];
  }

  return NULL;
}

void NewResource::setMainGlobalPath(const Directory& directory)
{
  NewResource::_mainGlobalPath = directory;
  NewResource::_mainGlobalPath.open();
}

void NewResource::addGlobalPath(const Directory& directory)
{
  std::vector<Directory>::const_iterator it = std::find(NewResource::_globalPaths.begin(), NewResource::_globalPaths.end(), directory);
  if (it == NewResource::_globalPaths.end())
    NewResource::_globalPaths.push_back(directory);
}



void NewResource::addResource(NewResource::StorePointer* pointer)
{
  this->_type->addResource(pointer);
}


void NewResource::registerType(NewResource::Type* type)
{
  NewResource::debug();
  if(type->id() == -1)
  {
    NewResource::_resourceTypes.push_back(type);
    type->setID(NewResource::_resourceTypes.size()-1);
  }
  NewResource::debug();
}

bool NewResource::addResourcePath(const std::string& resourceName, const std::string& pathName)
{
  std::vector<NewResource::Type*>::iterator it;
  for (it = NewResource::_resourceTypes.begin(); it != NewResource::_resourceTypes.end(); ++it)
    if (*(*it) == resourceName)
      return (*it)->addResourcePath(pathName);
  PRINTF(2)("ResourcePath %s could not be added to the ResourceType %s\n", pathName.c_str(), resourceName.c_str());
  return false;
}

bool NewResource::addResourceSubPath(const std::string& resourceName, const std::string& pathName)
{
  std::vector<NewResource::Type*>::iterator it;
  for (it = NewResource::_resourceTypes.begin(); it != NewResource::_resourceTypes.end(); ++it)
    if (*(*it) == resourceName)
      return (*it)->addResourceSubPath(pathName);
  PRINTF(2)("ResourceSubPath %s could not be added to the ResourceType %s\n", pathName.c_str(), resourceName.c_str());
  return false;
}



void NewResource::debug()
{
  PRINT(0)("==================================\n");
  PRINT(0)("DEBUG output for all Resources\n");
  PRINT(0)("==================================\n");

  PRINT(0)("Listing %d Types: \n", NewResource::_resourceTypes.size());
  std::vector<NewResource::Type*>::iterator it;
  for (it = NewResource::_resourceTypes.begin(); it != NewResource::_resourceTypes.end(); ++it)
    PRINT(0)("ResourceType '%s'\n", (*it)->storedClassID().name().c_str());

  PRINT(0)("==================================\n");

}





std::vector<std::string> NewResource::KeepLevel::_keepLevelNames;
void NewResource::KeepLevel::defineKeepLevelName(unsigned int level, const std::string& name)
{
  if (_keepLevelNames.size() <= level)
    _keepLevelNames.resize(level+1);
  _keepLevelNames[level] = name;
}


NewResource::Type::Type(const ClassID& classID)
    : _id(-1), _classID(classID)
{
  NewResource::registerType(this);
}

void NewResource::Type::addResource(NewResource::StorePointer* resource)
{
  this->_storedResources.push_back(resource);

}

bool NewResource::Type::addResourcePath(const std::string& path)
{
  std::vector<Directory>::const_iterator it;
  for (it = this->_resourcePaths.begin(); it != this->_resourcePaths.end(); ++it)
    if ((*it) == path)
      return false;
  this->_resourcePaths.push_back(path);
  return true;

}

bool NewResource::Type::addResourceSubPath(const std::string& subPath)
{
  std::vector<Directory>::const_iterator it;
  for (it = this->_resourceSubPaths.begin(); it != this->_resourceSubPaths.end(); ++it)
    if ((*it) == subPath)
      return false;
  this->_resourceSubPaths.push_back(subPath);
  return true;
}



NewResource::StorePointer::StorePointer(const std::string& loadString, const NewResource::KeepLevel& keeplevel)
    : _loadString(loadString), _keepLevel(keeplevel)
{}


void NewResource::Type::setID(int id)
{
  this->_id = id;
}
